from .CanvasClass import Canvas
from .DotClass import Dot
from .RectangleClass import Rectangle
from .TriangleClass import Triangle
from .SquareClass import Square
from .RhombClass import Rhomb
from .CircleClass import Circle
from .LineClass import Line

from .Settings import default_font_color, default_background_color, font_color, background_color
